package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.entity.IndexBaseMonthData;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;
import org.apache.poi.ss.formula.functions.Index;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;

public interface IndexBaseMonthDataDao extends BaseDAO<Long, IndexBaseMonthData> {

    int batchInsert(@Param("datas") List<IndexBaseMonthData> datas);

    int batchInsertWithDataType(@Param("datas") List<IndexBaseMonthData> datas,
                                @Param("dataType")IndexDataType dataType);

    int deleteWithMonthAndType(@Param("months") Collection months,
                               @Param("dataType")IndexDataType dataType);

    IndexBaseMonthData getOrganDataWithDayAndDataType(@Param("organId") Integer organId,
                                                      @Param("day") String day,
                                                      @Param("dataType")IndexDataType dataType);

    /**
     * @describe 获取首页统计数据
     * @author Joburgess
     * @date 2021/1/8 0008
     * @param organIds:
     * @param dataTypes:
     * @param startMonth:
     * @param endMonth:
     * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
     */
	List<IndexBaseMonthData> getIndexBaseData(@Param("organIds") Set<Integer> organIds,
                                              @Param("dataTypes") Set<String> dataTypes,
                                              @Param("startMonth") String startMonth,
                                              @Param("endMonth") String endMonth);

	List<IndexBaseMonthData> getWithDayAndDataType(@Param("day") String day,
                                                   @Param("dataType")IndexDataType dataType);

	/**
	 * @describe 统计学员注册数据
	 * @author Joburgess
	 * @date 2021/2/24 0024
	 * @param dayStr:
	 * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
	 */
	List<IndexBaseMonthData> getStudentRegistrationData(@Param("dayStr") String dayStr);

	/**
	 * @describe 统计新增学员注册数据
	 * @author Joburgess
	 * @date 2021/2/24 0024
	 * @param dayStr:
	 * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
	 */
	List<IndexBaseMonthData> getAddStudentRegistrationData(@Param("dayStr") String dayStr);

	/**
	 * @describe 付费转化率
	 * @author Joburgess
	 * @date 2021/2/24 0024
	 * @param dayStr:
	 * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
	 */
	List<IndexBaseMonthData> getChargeStudentChangeData(@Param("dayStr") String dayStr);

    /**
     * @describe 统计系统中指定时间段的学员注册数据
     * @author Joburgess
     * @date 2021/1/7 0007
     * @return java.util.List<com.ym.mec.biz.dal.dto.IndexBaseMonthDto>
     */
    List<IndexBaseMonthData> getStudentSignUpData(@Param("dayStr") String dayStr);

    /**
     * @describe 统计作业布置数据
     * @author Joburgess
     * @date 2021/1/7 0007
     * @return java.util.List<com.ym.mec.biz.dal.dto.IndexBaseMonthDto>
     */
    List<IndexBaseMonthData> getHomeworkData(@Param("dayStr") String dayStr,
                                             @Param("type") String type);

    /**
     * @describe 统计合作单位数据
     * @author Joburgess
     * @date 2021/1/11 0011
     * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
     */
    List<IndexBaseMonthData> getSchoolData(@Param("dayStr") String dayStr);

    /**
     * @describe 统计乐团数据
     * @author Joburgess
     * @date 2021/1/11 0011
     * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
     */
    List<IndexBaseMonthData> getMusicData(@Param("dayStr") String dayStr);

    /**
     * @describe 统计乐团学员数据
     * @author Joburgess
     * @date 2021/1/11 0011
     * @return java.util.List<com.ym.mec.biz.dal.entity.IndexBaseMonthData>
     */
    List<IndexBaseMonthData> getMusicStudentData(@Param("dayStr") String dayStr,
                                                 @Param("type") String type);

    List<IndexBaseMonthData> getMusicGroupPreRegistrationStudentData(@Param("dayStr") String dayStr);

    List<IndexBaseMonthData> getMusicGroupStudentFromPreData(@Param("dayStr") String dayStr,
                                                             @Param("paymentStatus") PaymentStatusEnum paymentStatus);

    List<Integer> getMusicGroupStudentIdFromPre(@Param("dayStr") String dayStr,
                                                @Param("paymentStatus") PaymentStatusEnum paymentStatus);

    List<IndexBaseMonthData> getStudentConversionData(@Param("dayStr") String dayStr,
                                                      @Param("studentIds") List<Integer> studentIds);

    List<IndexBaseMonthData> getOtherStudentData(@Param("dayStr") String dayStr);

    List<IndexBaseMonthData> getVipPracticeAddStudentData(@Param("dayStr") String dayStr);

    List<IndexBaseMonthData> getTeacherData(@Param("dayStr") String dayStr,
                                            @Param("jobNature") JobNatureEnum jobNature,
                                            @Param("isDemission") Boolean isDemission);

    List<IndexBaseMonthData> getGroupCourseData(@Param("dayStr") String dayStr,
                                                @Param("groupType")GroupType groupType,
                                                @Param("courseStatus") CourseStatusEnum courseStatus);

    List<IndexBaseMonthData> getGroupSurplusCourseData(@Param("dayStr") String dayStr,
                                                @Param("groupType")GroupType groupType,
                                                @Param("courseStatus") CourseStatusEnum courseStatus);

    int countLessThenThreeClassGroupNum(@Param("organIds") Set<Integer> organIds);
    List<String> getLessThenThreeMusicGroup(@Param("organIds") Set<Integer> organIds);

    int queryErrInspection(@Param("organIds") Set<Integer> organIds, @Param("startTime") String startTime);

    int countNoPaymentStudentNum(@Param("organIds") Set<Integer> organIds);
    List<String> getNoPaymentMusicGroup(@Param("organIds") Set<Integer> organIds);

    List<Long> queryApplyForQuitGroupNum(@Param("organIds") Set<Integer> organIds);

    int queryStudentAttendanceInfo(@Param("organIds") Set<Integer> organIds,
                                          @Param("type") String type, @Param("startTime") String startTime);


    /**
     * 获取考勤异常编号
     * @param organIds
     * @return
     */
    int getAttendanceError(@Param("organIds") String organIds,@Param("startTime") String startTime);

    /**
     * 获取旷课考勤
     * @param organIds
     * @return
     */
    int getNoAttendance(@Param("organIds") String organIds, @Param("startTime") String startTime);

    /**
     * 获取老师离职申请数
     * @param organIdsStr
     * @return
     */
    int queryTeacherLeave(@Param("organIdsStr") String organIdsStr, @Param("startTime") String startTime);

    /**
     * 获取预计课酬偏低的老师编号
     * @param organIdsStr
     * @param monthStr
     * @return
     */
    int queryLowTeacherSalary(@Param("organIdsStr") String organIdsStr, @Param("monthStr") String monthStr);

    /**
     * 乐团巡查任务未计划
     * @param organIdsStr
     * @return
     */
    int queryInspectionItem(@Param("organIdsStr") String organIdsStr, @Param("startTime") String startTime);

    /**
     * 乐团巡查任务未提交
     * @param organIdsStr
     * @return
     */
    int queryInspectionItemPlan(@Param("organIdsStr") String organIdsStr, @Param("startTime") String startTime);

    /**
     * 未完成的回访任务
     * @param organIdsStr
     * @return
     */
    int queryStudentVisit(@Param("organIdsStr") String organIdsStr, @Param("startTime") String startTime);

    List<IndexBaseMonthData> getFinancePayData(@Param("dayStr") String dayStr);

    List<IndexBaseMonthData> getFinanceBalanceData(@Param("dayStr") String dayStr);

    List<IndexBaseMonthData> getFinanceActualData(@Param("dayStr") String dayStr);
}
