package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.IndexBaseDto;
import com.ym.mec.biz.dal.entity.IndexErrInfoDto;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.IndexDataQueryInfo;
import com.ym.mec.common.constant.CommonConstants;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import com.ym.mec.biz.dal.entity.IndexBaseMonthData;
import com.ym.mec.biz.service.IndexBaseMonthDataService;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.IndexErrorType.WAIT_CREATE_PAYMENT_CALENDER;

@Service
public class IndexBaseMonthDataServiceImpl extends BaseServiceImpl<Long, IndexBaseMonthData> implements IndexBaseMonthDataService {
	
	@Autowired
	private IndexBaseMonthDataDao indexBaseMonthDataDao;
	@Autowired
	private OrganizationDao organizationDao;
	@Autowired
	private EmployeeDao employeeDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private SysMessageDao sysMessageDao;

	private static ThreadLocal<Set<Integer>> organIds = new ThreadLocal<Set<Integer>>(){
		@Override
		protected Set<Integer> initialValue() {
			return new HashSet<>();
		}
	};

	@Override
	public BaseDAO<Long, IndexBaseMonthData> getDAO() {
		return indexBaseMonthDataDao;
	}

	@Override
	public List<IndexBaseDto> getIndexBaseData(IndexDataQueryInfo queryInfo) {
		List<IndexBaseDto> result = new ArrayList<>();

		LocalDate nowDate = LocalDate.now();
//		Date currentMonth = Date.from(nowDate.atStartOfDay(DateUtil.zoneId).toInstant());

		LocalDate startDate = nowDate.plusDays(-30);
		if(Objects.nonNull(queryInfo.getStartDate())){
			startDate = LocalDateTime.ofInstant(queryInfo.getStartDate().toInstant(), DateUtil.zoneId).toLocalDate();
		}

		LocalDate endDate = nowDate.plusDays(-1);
		if(Objects.nonNull(queryInfo.getEndDate())){
			endDate = LocalDateTime.ofInstant(queryInfo.getEndDate().toInstant(), DateUtil.zoneId).toLocalDate();
		}
		Date currentMonth = Date.from(endDate.atStartOfDay(DateUtil.zoneId).toInstant());

		Set<Integer> organIds = null;
		if(StringUtils.isNotBlank(queryInfo.getOrganId())){
			organIds = Arrays.stream(queryInfo.getOrganId().split(",")).map(Integer::new).collect(Collectors.toSet());
		}
		Set<String> dataTypes = null;
		if(StringUtils.isNotBlank(queryInfo.getDataTypes())){
			dataTypes = Arrays.stream(queryInfo.getDataTypes().split(",")).collect(Collectors.toSet());
		}

		List<IndexBaseMonthData> indexBaseDatas = indexBaseMonthDataDao.getIndexBaseData(organIds, dataTypes, startDate.toString(), endDate.toString());
		if(Objects.isNull(indexBaseDatas)){
			indexBaseDatas = new ArrayList<>();
		}

		Map<IndexDataType, List<IndexBaseMonthData>> typeDateMap = indexBaseDatas.stream().filter(d->Objects.nonNull(d.getDataType())).collect(Collectors.groupingBy(IndexBaseMonthData::getDataType));

		for (IndexDataType dataType : IndexDataType.values()) {
			if(typeDateMap.containsKey(dataType)){
				continue;
			}

			IndexBaseMonthData indexBaseMonthData = new IndexBaseMonthData();
			indexBaseMonthData.setMonth(Date.from(endDate.atStartOfDay(DateUtil.zoneId).toInstant()));
			indexBaseMonthData.setTotalNum(BigDecimal.ZERO);
			indexBaseMonthData.setActivateNum(BigDecimal.ZERO);
			indexBaseMonthData.setPercent(BigDecimal.ZERO);
			indexBaseMonthData.setDataType(dataType);
			typeDateMap.put(dataType, new ArrayList<>(Arrays.asList(indexBaseMonthData)));
		}

		for (Map.Entry<IndexDataType, List<IndexBaseMonthData>> typeDateMapEntry : typeDateMap.entrySet()) {
			Set<String> hasMonths = typeDateMapEntry.getValue().stream().map(d -> DateUtil.dateToString(d.getMonth(), "yyyy-MM-dd")).collect(Collectors.toSet());
			LocalDate currentMonthDate = startDate;
			while (currentMonthDate.compareTo(endDate)<=0){
				if(hasMonths.contains(currentMonthDate.toString())){
					currentMonthDate = currentMonthDate.plusDays(1);
					continue;
				}
				IndexBaseMonthData indexBaseMonthData = new IndexBaseMonthData();
				indexBaseMonthData.setMonth(Date.from(currentMonthDate.atStartOfDay(DateUtil.zoneId).toInstant()));
				indexBaseMonthData.setTotalNum(BigDecimal.ZERO);
				indexBaseMonthData.setActivateNum(BigDecimal.ZERO);
				indexBaseMonthData.setPercent(BigDecimal.ZERO);
				indexBaseMonthData.setDataType(typeDateMapEntry.getKey());
				typeDateMapEntry.getValue().add(indexBaseMonthData);
				currentMonthDate = currentMonthDate.plusDays(1);
			}
			typeDateMapEntry.getValue().sort(Comparator.comparing(IndexBaseMonthData::getMonth));
			if(typeDateMapEntry.getKey().getPercent()){
				for (IndexBaseMonthData indexBaseMonthData : typeDateMapEntry.getValue()) {
					if(indexBaseMonthData.getTotalNum().compareTo(BigDecimal.ZERO)==0){
						indexBaseMonthData.setPercent(BigDecimal.ZERO);
						continue;
					}
					indexBaseMonthData.setPercent(indexBaseMonthData.getActivateNum().divide(indexBaseMonthData.getTotalNum(), CommonConstants.DECIMAL_PLACE, BigDecimal.ROUND_DOWN).multiply(new BigDecimal(100)).setScale(CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_DOWN));
				}
			}
			IndexBaseDto indexBaseData = new IndexBaseDto(typeDateMapEntry.getKey(),typeDateMapEntry.getKey().getMsg());
			indexBaseData.setIndexMonthData(typeDateMapEntry.getValue(), currentMonth);
			if(IndexDataType.FINANCE_PAY.equals(typeDateMapEntry.getKey()) || IndexDataType.FINANCE_AMOUNT.equals(typeDateMapEntry.getKey()) ||
				IndexDataType.FINANCE_BALANCE_AMOUNT.equals(typeDateMapEntry.getKey())
				||IndexDataType.MUSIC_GROUP_COURSE.equals(typeDateMapEntry.getKey())||IndexDataType.VIP_GROUP_COURSE.equals(typeDateMapEntry.getKey())
					||IndexDataType.PRACTICE_GROUP_COURSE.equals(typeDateMapEntry.getKey())){
				indexBaseData.setPercent(typeDateMapEntry.getValue().stream().map(IndexBaseMonthData::getPercent).reduce(BigDecimal.ZERO, BigDecimal::add));
			}
			result.add(indexBaseData);
		}
		return result;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void indexBaseDataTask(String dayStr) {
		LocalDate nowDate = LocalDate.now();

		List<Organization> allOrgans = organizationDao.findAllOrgans();
		if(CollectionUtils.isEmpty(this.organIds.get())){
			this.organIds.get().clear();
			this.organIds.get().addAll(allOrgans.stream().map(Organization::getId).collect(Collectors.toSet()));
		}

		if(StringUtils.isBlank(dayStr)){
			dayStr = nowDate.plusDays(-1).toString();
		}

		LocalDate day = LocalDate.parse(dayStr, DateUtil.dateFormatter);
		LocalDate monday = day.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());

		//学员数据
		saveData(indexBaseMonthDataDao.getStudentRegistrationData(dayStr), dayStr, IndexDataType.STUDENT_REGISTRATION_NUM);
		saveData(indexBaseMonthDataDao.getChargeStudentChangeData(dayStr), dayStr, IndexDataType.CHARGE_STUDENT_CHANGE_RATE);
		saveData(indexBaseMonthDataDao.getStudentSignUpData(dayStr), dayStr, IndexDataType.ACTIVATION_RATE);

		//运营数据
		saveData(indexBaseMonthDataDao.getSchoolData(dayStr), dayStr, IndexDataType.SCHOOL);
		saveData(indexBaseMonthDataDao.getMusicData(dayStr), dayStr, IndexDataType.PROGRESS_MUSIC_GROUP_NUM);

		//人事数据
		saveData(indexBaseMonthDataDao.getTeacherData(dayStr, null, null), dayStr, IndexDataType.TEACHER_NUM);
		saveData(indexBaseMonthDataDao.getTeacherData(dayStr, JobNatureEnum.FULL_TIME, null), dayStr, IndexDataType.FULL_TIME_NUM);
		saveData(indexBaseMonthDataDao.getTeacherData(dayStr, JobNatureEnum.PART_TIME, null), dayStr, IndexDataType.PART_TIME_NUM);
//		saveData(indexBaseMonthDataDao.getTeacherData(dayStr, null, true), dayStr, IndexDataType.DIMISSION_NUM);

		//剩余课时
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, null, CourseStatusEnum.OVER), dayStr, IndexDataType.SURPLUS_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.MUSIC, CourseStatusEnum.OVER), dayStr, IndexDataType.SURPLUS_MUSIC_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.VIP, CourseStatusEnum.OVER), dayStr, IndexDataType.SURPLUS_VIP_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.PRACTICE, CourseStatusEnum.OVER), dayStr, IndexDataType.SURPLUS_PRACTICE_COURSE_NUM);

		//已消耗课时
		saveData(indexBaseMonthDataDao.getGroupSurplusCourseData(dayStr, null, CourseStatusEnum.OVER), dayStr, IndexDataType.OVER_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupSurplusCourseData(dayStr, GroupType.MUSIC, CourseStatusEnum.OVER), dayStr, IndexDataType.OVER_MUSIC_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupSurplusCourseData(dayStr, GroupType.VIP, CourseStatusEnum.OVER), dayStr, IndexDataType.OVER_VIP_COURSE_NUM);
		saveData(indexBaseMonthDataDao.getGroupSurplusCourseData(dayStr, GroupType.PRACTICE, CourseStatusEnum.OVER), dayStr, IndexDataType.OVER_PRACTICE_COURSE_NUM);

		//经营数据
		saveData(indexBaseMonthDataDao.getFinancePayData(dayStr),dayStr,IndexDataType.FINANCE_PAY);
		saveData(indexBaseMonthDataDao.getFinanceBalanceData(dayStr),dayStr,IndexDataType.FINANCE_BALANCE_AMOUNT);
		saveData(indexBaseMonthDataDao.getFinanceActualData(dayStr),dayStr,IndexDataType.FINANCE_AMOUNT);

		//业务数据
		saveData(indexBaseMonthDataDao.getHomeworkData(dayStr, null), monday.toString(), IndexDataType.HOMEWORK_CREATE_RATE);
		saveData(indexBaseMonthDataDao.getHomeworkData(dayStr, "submit"), monday.toString(), IndexDataType.HOMEWORK_SUBMIT_RATE);
		saveData(indexBaseMonthDataDao.getHomeworkData(dayStr, "comment"), monday.toString(), IndexDataType.HOMEWORK_COMMENT_RATE);

		//课程数据
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.MUSIC, CourseStatusEnum.OVER), dayStr, IndexDataType.MUSIC_GROUP_COURSE);
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.VIP, CourseStatusEnum.OVER), dayStr, IndexDataType.VIP_GROUP_COURSE);
		saveData(indexBaseMonthDataDao.getGroupCourseData(dayStr, GroupType.PRACTICE, CourseStatusEnum.OVER), dayStr, IndexDataType.PRACTICE_GROUP_COURSE);

		//学员变动
		saveData(indexBaseMonthDataDao.getAddStudentRegistrationData(dayStr), dayStr, IndexDataType.ADD_STUDENT_REGISTRATION_NUM);
		saveData(indexBaseMonthDataDao.getMusicStudentData(dayStr, null), dayStr, IndexDataType.MUSIC_GROUP_STUDENT);
		saveData(null, dayStr, IndexDataType.NEWLY_STUDENT_NUM);
		saveData(null, dayStr, IndexDataType.QUIT_MUSIC_GROUP_STUDENT_NUM);
		saveData(indexBaseMonthDataDao.getOtherStudentData(dayStr), dayStr, IndexDataType.VIP_PRACTICE_STUDENT_NUM);
		saveData(indexBaseMonthDataDao.getVipPracticeAddStudentData(dayStr), dayStr, IndexDataType.VIP_PRACTICE_ADD_STUDENT_NUM);
	}

	/**
	 * @describe 处理并保存每日数据
	 * @author Joburgess
	 * @date 2021/1/11 0011
	 * @return void
	 */
	private void saveData(List<IndexBaseMonthData> dataList, String dayStr, IndexDataType indexDataType){
		Date date = DateUtil.stringToDate(dayStr, "yyyy-MM-dd");
		if(Objects.isNull(dataList)){
			dataList = new ArrayList<>();
		}
		Set<Integer> hasOrganIds = dataList.stream().map(IndexBaseMonthData::getOrganId).collect(Collectors.toSet());

		boolean specialType = IndexDataType.NEWLY_STUDENT_NUM.equals(indexDataType)||IndexDataType.QUIT_MUSIC_GROUP_STUDENT_NUM.equals(indexDataType);
		Set<Integer> ignoreOrganIds = new HashSet<>();
		if(specialType){
			List<IndexBaseMonthData> indexBaseDatas = indexBaseMonthDataDao.getWithDayAndDataType(dayStr, indexDataType);
			if(!CollectionUtils.isEmpty(indexBaseDatas)) {
				ignoreOrganIds = indexBaseDatas.stream().map(IndexBaseMonthData::getOrganId).collect(Collectors.toSet());
			}
		}

		for (Integer organId : this.organIds.get()) {
			if(hasOrganIds.contains(organId)){
				continue;
			}
			if(specialType&&ignoreOrganIds.contains(organId)){
				continue;
			}
			dataList.add(new IndexBaseMonthData(date, organId));
		}
		if(!IndexDataType.NEWLY_STUDENT_NUM.equals(indexDataType)&&!IndexDataType.QUIT_MUSIC_GROUP_STUDENT_NUM.equals(indexDataType)){
			indexBaseMonthDataDao.deleteWithMonthAndType(Arrays.asList(dayStr), indexDataType);
		}
		if(!CollectionUtils.isEmpty(dataList)){
			indexBaseMonthDataDao.batchInsertWithDataType(dataList, indexDataType);
		}
	}

	@Override
	public Map<String, Object> getIndexErrData(String organIdsStr, IndexErrorType errorType) {
		//只筛选指定时间之后的数据
		String startTime = DateUtil.format(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(new Date(), -2)),DateUtil.ISO_EXPANDED_DATE_FORMAT);

		Set<Integer> organIds = null;
		if(StringUtils.isNotBlank(organIdsStr)){
			organIds = Arrays.stream(organIdsStr.split(",")).map(Integer::new).collect(Collectors.toSet());
		}
		Date date = new Date();

		List<IndexErrInfoDto> all = new ArrayList<>();

		if(IndexErrorType.MUSIC_PATROL.equals(errorType)){
			IndexErrInfoDto<IndexErrInfoDto> one = new IndexErrInfoDto<>();
			one.setErrorType(IndexErrorType.MUSIC_PATROL);
			one.setDesc(IndexErrorType.MUSIC_PATROL.getMsg());
			List<IndexErrInfoDto> oneChild = new ArrayList<>();

			oneChild.add(new IndexErrInfoDto(IndexErrorType.HIGH_CLASS_STUDENT_LESS_THAN_THREE, IndexErrorType.HIGH_CLASS_STUDENT_LESS_THAN_THREE.getMsg(), indexBaseMonthDataDao.countLessThenThreeClassGroupNum(organIds), null));
			int errInspection = indexBaseMonthDataDao.queryErrInspection(organIds,startTime);
			oneChild.add(new IndexErrInfoDto(IndexErrorType.MUSIC_PATROL_ITEM, IndexErrorType.MUSIC_PATROL_ITEM.getMsg(),errInspection, null));

			one.setNum(oneChild.stream().mapToInt(IndexErrInfoDto::getNum).sum());
			one.setResult(oneChild);
			all.add(one);
		}

		if(IndexErrorType.STUDENT_INFO.equals(errorType)){

			IndexErrInfoDto<IndexErrInfoDto> two = new IndexErrInfoDto<>();
			two.setErrorType(IndexErrorType.STUDENT_INFO);
			two.setDesc(IndexErrorType.STUDENT_INFO.getMsg());
			List<IndexErrInfoDto> twoChild = new ArrayList<>();

			twoChild.add(new IndexErrInfoDto(IndexErrorType.STUDENT_NOT_PAYMENT, IndexErrorType.STUDENT_NOT_PAYMENT.getMsg(), indexBaseMonthDataDao.countNoPaymentStudentNum(organIds), indexBaseMonthDataDao.getNoPaymentMusicGroup(organIds)));
			List<Long> quitGroupNum = indexBaseMonthDataDao.queryApplyForQuitGroupNum(organIds);
			twoChild.add(new IndexErrInfoDto(IndexErrorType.STUDENT_APPLY_FOR_QUIT_MUSIC_GROUP, IndexErrorType.STUDENT_APPLY_FOR_QUIT_MUSIC_GROUP.getMsg(), quitGroupNum.size(),  quitGroupNum));
//			int attendanceInfo = indexBaseMonthDataDao.queryStudentAttendanceInfo(organIds, StudentAttendanceStatusEnum.TRUANT.getCode(),startTime);
//			twoChild.add(new IndexErrInfoDto(IndexErrorType.COURSE_TRUANT_STUDENT_NUM, IndexErrorType.COURSE_TRUANT_STUDENT_NUM.getMsg(), attendanceInfo, null));
//			int attendanceInfo1 = indexBaseMonthDataDao.queryStudentAttendanceInfo(organIds, StudentAttendanceStatusEnum.LEAVE.getCode(),startTime);
//			twoChild.add(new IndexErrInfoDto(IndexErrorType.COURSE_LEAVE_STUDENT_NUM, IndexErrorType.COURSE_LEAVE_STUDENT_NUM.getMsg(), attendanceInfo1, null));
			two.setNum(twoChild.stream().mapToInt(IndexErrInfoDto::getNum).sum());
			two.setResult(twoChild);
			all.add(two);
		}

		if(IndexErrorType.TEACHER_INFO.equals(errorType)){
			IndexErrInfoDto<IndexErrInfoDto> three = new IndexErrInfoDto<>();
			three.setErrorType(IndexErrorType.TEACHER_INFO);
			three.setDesc(IndexErrorType.TEACHER_INFO.getMsg());
			List<IndexErrInfoDto> threeChild = new ArrayList<>();

			int attendanceError = indexBaseMonthDataDao.getAttendanceError(organIdsStr,startTime);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.TEACHER_EXCEPTION_ATTENDANCE, IndexErrorType.TEACHER_EXCEPTION_ATTENDANCE.getMsg(), attendanceError, null));

			int noAttendance = indexBaseMonthDataDao.getNoAttendance(organIdsStr,startTime);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.TEACHER_NOT_A_CLASS, IndexErrorType.TEACHER_NOT_A_CLASS.getMsg(), noAttendance, null));

//			int teacherLeave = indexBaseMonthDataDao.queryTeacherLeave(organIdsStr,startTime);
//			threeChild.add(new IndexErrInfoDto(IndexErrorType.TEACHER_LEAVE, IndexErrorType.TEACHER_LEAVE.getMsg(),teacherLeave, null));

			String monthStr = DateUtil.format(date, DateUtil.ISO_YEAR_MONTH_FORMAT);
			//全职未离职老师
			int lowTeacherSalary = indexBaseMonthDataDao.queryLowTeacherSalary(organIdsStr,monthStr);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.TEACHER_EXPECT_SALARY_BE_LOW, IndexErrorType.TEACHER_EXPECT_SALARY_BE_LOW.getMsg(),lowTeacherSalary, null));

			int inspectionItem = indexBaseMonthDataDao.queryInspectionItem(organIdsStr,startTime);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.INSPECTION_ITEM, IndexErrorType.INSPECTION_ITEM.getMsg(),inspectionItem, null));

			int inspectionItemPlan = indexBaseMonthDataDao.queryInspectionItemPlan(organIdsStr,startTime);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.INSPECTION_ITEM_PLAN, IndexErrorType.INSPECTION_ITEM_PLAN.getMsg(),inspectionItemPlan, null));

			int studentVisit = indexBaseMonthDataDao.queryStudentVisit(organIdsStr,startTime);
			threeChild.add(new IndexErrInfoDto(IndexErrorType.STUDENT_VISIT, IndexErrorType.STUDENT_VISIT.getMsg(),studentVisit, null));

			three.setNum(threeChild.stream().mapToInt(IndexErrInfoDto::getNum).sum());
			three.setResult(threeChild);
			all.add(three);
		}

		Map<String, Object> result = new HashMap<>(2);
		result.put("totalNum", all.stream().mapToInt(IndexErrInfoDto::getNum).sum());
		result.put("data", all);
		return result;
	}

	@Override
	public List<IndexErrInfoDto> getRemindMatterData(String organIds) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		//当前用户是否是分部经理
		List<Integer> userRole = employeeDao.queryUserRole(sysUser.getId());
		//获取关联的乐团列表
		List<String> musicGroupIds = musicGroupDao.queryIdsByEduIdAndOrganIds(userRole.contains(3)?sysUser.getId():null,organIds);
		String configValue1 = sysConfigDao.findConfigValue("push_create_payment_calender");
		Date date = new Date();
		String format = DateUtil.format(date, DateUtil.DEFAULT_PATTERN);
		List<IndexErrInfoDto> result = new ArrayList<>();
		IndexErrInfoDto indexErrInfoDto = new IndexErrInfoDto();
		indexErrInfoDto.setErrorType(WAIT_CREATE_PAYMENT_CALENDER);
		indexErrInfoDto.setDesc(WAIT_CREATE_PAYMENT_CALENDER.getMsg());
		if(musicGroupIds.size() > 0){
			List<Long> calenderIds = musicGroupPaymentCalenderDao.queryEndIds(configValue1, format,musicGroupIds);
			if(calenderIds.size() > 0){
				List<String> musicGroupIdList = musicGroupPaymentCalenderDao.queryMusicGroupIds(calenderIds);
				indexErrInfoDto.setResult(musicGroupIdList);
			}
			indexErrInfoDto.setNum(calenderIds.size());
		}
		result.add(indexErrInfoDto);
		return result;
	}

	@Override
	public Map<String,Boolean> hasIndexErrData(String organIdsStr) {
		String startTime = DateUtil.format(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(new Date(), -2)),DateUtil.ISO_EXPANDED_DATE_FORMAT);

		Map<String,Boolean> resultMap = new HashMap<>();
		Set<Integer> organIds = null;
		if(StringUtils.isNotBlank(organIdsStr)){
			organIds = Arrays.stream(organIdsStr.split(",")).map(Integer::new).collect(Collectors.toSet());
		}
		Date date = new Date();
		boolean flag1 = false;
		int countLessThenThreeClassGroupNum = indexBaseMonthDataDao.countLessThenThreeClassGroupNum(organIds);
		if(countLessThenThreeClassGroupNum > 0){
			flag1 = true;
		}
		if(!flag1){
			int errInspection = indexBaseMonthDataDao.queryErrInspection(organIds,startTime);
			if(errInspection > 0){
				flag1 = true;
			}
		}
		resultMap.put("musicPatrol",flag1);
		boolean flag2 = false;
		if(!flag2){
			int countNoPaymentStudentNum = indexBaseMonthDataDao.countNoPaymentStudentNum(organIds);
			if(countNoPaymentStudentNum > 0){
				flag2 = true;
			}
		}
		if(!flag2){
			List<Long> quitGroupNum = indexBaseMonthDataDao.queryApplyForQuitGroupNum(organIds);
			if(quitGroupNum.size() > 0){
				flag2 = true;
			}
		}
		/*if(!flag2){
			int attendanceInfo = indexBaseMonthDataDao.queryStudentAttendanceInfo(organIds, StudentAttendanceStatusEnum.TRUANT.getCode(),startTime);
			if(attendanceInfo > 0){
				flag2 = true;
			}
		}
		if(!flag2){
			int attendanceInfo1 = indexBaseMonthDataDao.queryStudentAttendanceInfo(organIds, StudentAttendanceStatusEnum.LEAVE.getCode(),startTime);
			if(attendanceInfo1 > 0){
				flag2 = true;
			}
		}*/
		resultMap.put("studentInfo",flag2);
		boolean flag3 = false;
		if(!flag3){
			int attendanceError = indexBaseMonthDataDao.getAttendanceError(organIdsStr,startTime);
			if(attendanceError > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			int noAttendance = indexBaseMonthDataDao.getNoAttendance(organIdsStr,startTime);
			if(noAttendance > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			int teacherLeave = indexBaseMonthDataDao.queryTeacherLeave(organIdsStr,startTime);
			if(teacherLeave > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			String monthStr = DateUtil.format(date, DateUtil.ISO_YEAR_MONTH_FORMAT);
			int lowTeacherSalary = indexBaseMonthDataDao.queryLowTeacherSalary(organIdsStr,monthStr);
			if(lowTeacherSalary > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			int inspectionItem = indexBaseMonthDataDao.queryInspectionItem(organIdsStr,startTime);
			if(inspectionItem > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			int inspectionItemPlan = indexBaseMonthDataDao.queryInspectionItemPlan(organIdsStr,startTime);
			if(inspectionItemPlan > 0){
				flag3 = true;
			}
		}
		if(!flag3){
			int studentVisit = indexBaseMonthDataDao.queryStudentVisit(organIdsStr,startTime);
			if(studentVisit > 0){
				flag3 = true;
			}
		}
		resultMap.put("teacherInfo",flag3);
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		//当前用户是否是分部经理
		List<Integer> userRole = employeeDao.queryUserRole(sysUser.getId());
		//获取关联的乐团列表
		List<String> musicGroupIds = musicGroupDao.queryIdsByEduIdAndOrganIds(userRole.contains(3)?sysUser.getId():null,organIdsStr);
		String configValue1 = sysConfigDao.findConfigValue("push_create_payment_calender");
		date = new Date();
		String format = DateUtil.format(date, DateUtil.DEFAULT_PATTERN);
		if(musicGroupIds.size() > 0){
			List<Long> calenderIds = musicGroupPaymentCalenderDao.queryEndIds(configValue1, format,musicGroupIds);
			if(calenderIds.size() > 0){
				resultMap.put("remindMatterData",true);
			}else {
				resultMap.put("remindMatterData",false);
			}
		}
		return resultMap;
	}
}
